@@ -84,6 +84,10 @@ def build_info():
         _figure_linux_info (info)
     elif info['system'] == 'Darwin':
         _figure_macos_info (info)
+    elif info['system'] == 'FreeBSD':
+        _figure_freebsd_info (info)
+    elif info['system'] == 'SunOS':
+        _figure_solaris_info (info)
 
     # Users and Groups
     if os.access ('/etc/group', os.R_OK):
@@ -133,6 +137,17 @@ def _figure_macos_info (info):
     if port_bin:
         info['macports_path'] = '/'.join (port_bin.split('/')[:-1])
 
+
+def _figure_freebsd_info (info):
+    path = '/usr/ports'
+    if os.path.isdir (path):
+        info['freebsd_ports_path'] = path
+
+
+def _figure_solaris_info (info):
+    None
+
+
 if __name__ == '__main__':
     import pprint
     pprint.pprint (get_info())
