@@ -64,7 +64,7 @@ def is_web_path (value):
     raise ValueError, _('Malformed path')
 
 def strip_trailing_slashes (value):
-    while value[-1] == '/':
+    while len(value) > 1 and  value[-1] == '/':
         value = value [:-1]
     return value
 
@@ -443,5 +443,3 @@ def has_no_double_quotes (value):
     if '"' in value:
         raise ValueError, _('Cannot contain the double quote (") character')
     return value
-
-
