@@ -9,7 +9,7 @@ vserver!1!rule!470!handler!url = http://www.cherokee-project.com
 
 class Test (TestBase):
     def __init__ (self):
-        TestBase.__init__ (self)
+        TestBase.__init__ (self, __file__)
         self.name = "Redir to URL with params"
 
         self.request          = "GET /redir47/something?param=1&param2 HTTP/1.0\r\n"
